<?php

namespace Drupal\pb_ai\Commands;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Extension\ModuleHandler;
use Drupal\user\Entity\User;
use Drush\Commands\DrushCommands;

/**
 * A Drush commandfile.
 *
 * In addition to this file, you need a drush.services.yml
 * in root of your module, and a composer.json file that provides the name
 * of the services file to use.
 *
 * See these files for an example of injecting services into this class.
 *
 * @package Drupal\pg_ai\Commands
 */
class PopulateModules extends DrushCommands {

  /**
   * The module handler.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * PopulateModules constructor.
   *
   * @param \Drupal\Core\Extension\ModuleHandler $moduleHandler
   *   The module handler.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   The entity type manager.
   */
  public function __construct(
    ModuleHandler $moduleHandler,
    EntityTypeManagerInterface $entityTypeManager,
  ) {
    $this->moduleHandler = $moduleHandler;
    $this->entityTypeManager = $entityTypeManager;
  }

  /**
   * Populate all the modules.
   *
   * @command project_browser:ai:populate
   * @aliases pbap
   * @usage project_browser:ai:populate
   *   Populates the node modules from the project browser.
   */
  public function populateModules() {
    $module_path = $this->moduleHandler->getModule('project_browser')->getPath();
    $modules = json_decode(file_get_contents($module_path . '/fixtures/project_data.json'), TRUE);
    foreach ($modules as $module) {
      $extra = unserialize($module['project_data']);
      $usage = 0;
      if (isset($extra['project_usage'])) {
        foreach ($extra['project_usage'] as $key => $value) {
          $usage += $value;
        }
      }
      if ($usage > 50) {
        $version = '';
        foreach ($extra['releases'] as $release) {
          if (str_contains($release['version'], 'rc')) {
            $version = 'rc';
          }
          elseif (str_contains($release['version'], 'beta') && $version != 'rc') {
            $version = 'beta';
          }
          elseif (str_contains($release['version'], 'alpha') && $version != 'rc' && $version != 'beta') {
            $version = 'alpha';
          }
          elseif (str_contains($release['version'], 'dev') && $version != 'rc' && $version != 'beta' && $version != 'alpha') {
            $version = 'dev';
          }
          else {
            $version = 'production';
            // Jump out of the loop.
            break;
          }
        }
        if ($version) {
          // If the module exists, update it. Search on link.
          $nodes = $this->entityTypeManager->getStorage('node')->loadByProperties(['field_module_link' => 'https://www.drupal.org/node/' . $extra['nid']]);
          if (count($nodes)) {
            // Update.
            $node = $nodes[key($nodes)];
            echo "Updated: " . $node->title->value . "\n";
          }
          else {
            exit;
            // Otherwise create.
            $node = $this->entityTypeManager->getStorage('node')->create([
              'type' => 'modules',
              'uid' => User::load(1)->id(),
            ]);
            echo "Created: " . $module['title'] . "\n";
          }
          $node->set('title', $module['title']);
          $node->set('field_module_description', $this->cleanupDescription($extra['body']['value']));
          $node->set('field_project_usage', $usage);
          $node->set('field_creation_date', $extra['created']);
          $node->set('field_data_name', $extra['field_project_machine_name']);
          $node->set('field_changed_date', $extra['changed']);
          $node->set('field_security_covered', $module['field_security_advisory_coverage'] == 'not-covered' ? 'not_covered_by_security_advisory' : 'covered_by_security_advisory');
          $node->set('field_version_available', $version);

          $node->save();
        }
      }
    }
  }

  /**
   * Cleanup description.
   *
   * @param string $description
   *   The description.
   *
   * @return string
   *   The cleaned up description.
   */
  protected function cleanupDescription($description) {
    $description = preg_replace('/\s+/', ' ', $description);
    // </p> to double new line.
    $description = str_replace('</p>', "\n\n", $description);
    // </h3> to single new line.
    $description = str_replace('</h3>', "\n", $description);
    // li to *.
    $description = str_replace('<li>', '* ', $description);
    $description = str_replace('</li>', "\n", $description);
    // </ol> or </ul> to new line.
    $description = str_replace(['</ol>', '</ul>'], "\n", $description);
    // <ol> or <ul> to new line.
    $description = str_replace(['<ol>', '<ul>'], "\n", $description);
    // Remove all tags.
    $description = strip_tags($description);

    return $description;
  }

  /**
   * Export all the modules.
   *
   * @command project_browser:ai:export
   * @aliases pbae
   * @usage project_browser:ai:export
   *   Exports the node modules from the project browser.
   */
  public function exportModules() {
    // Get all modules content types and iterate.
    $nodes = $this->entityTypeManager->getStorage('node')->loadByProperties(['type' => 'modules']);
    $allData = [];
    $i = 0;
    $t = 0;
    $module_path = $this->moduleHandler->getModule('pb_ai')->getPath();
    foreach ($nodes as $node) {
      $allData[] = [
        'title' => $node->title->value,
        'field_module_description' => $node->field_module_description->value,
        'field_module_link' => $node->field_module_link->uri,
        'field_creation_date' => $node->field_creation_date->value,
        'field_changed_date' => $node->field_changed_date->value,
        'field_data_name' => $node->field_data_name->value,
        'field_project_usage' => $node->field_project_usage->value,
        'field_security_covered' => $node->field_security_covered->value,
        'field_module_summary' => $node->field_module_summary->value,
        'field_affordances' => $node->field_affordances->value,
        'field_module_status' => $node->field_module_status->value,
        'field_module_author' => $node->field_drupal_author->value,
      ];
      $i++;
      if ($i > 500) {
        file_put_contents($module_path . '/resources/modules_' . $t . '.json', json_encode($allData));
        $t++;
        $i = 0;
        $allData = [];
      }
    }

    $t++;
    file_put_contents($module_path . '/resources/modules_' . $t . '.json', json_encode($allData));
  }

  /**
   * Import all the modules.
   *
   * @command project_browser:ai:import
   * @aliases pbai
   * @usage project_browser:ai:import
   *   Imports the node modules from the project browser.
   */
  public function importModules() {
    $module_path = $this->moduleHandler->getModule('pb_ai')->getPath();
    $files = glob($module_path . '/resources/*.json');
    foreach ($files as $file) {
      $modules = json_decode(file_get_contents($file), TRUE);
      foreach ($modules as $module) {

        // Check using data_name if the module exists.
        $nodes = $this->entityTypeManager->getStorage('node')->loadByProperties(['field_data_name' => $module['field_data_name']]);
        if (count($nodes)) {
          // Update.
          $node = $nodes[key($nodes)];
          echo "Updated: " . $node->title->value . "\n";
        }
        else {
          // Otherwise create.
          $node = $this->entityTypeManager->getStorage('node')->create([
            'type' => 'modules',
            'uid' => User::load(1)->id(),
          ]);
          echo "Created: " . $module['title'] . "\n";
        }
        $node->set('title', $module['title']);
        $node->set('field_module_description', $module['field_module_description']);
        $node->set('field_module_link', $module['field_module_link']);
        $node->set('field_creation_date', $module['field_creation_date']);
        $node->set('field_changed_date', $module['field_changed_date']);
        $node->set('field_data_name', $module['field_data_name']);
        $node->set('field_project_usage', $module['field_project_usage']);
        $node->set('field_security_covered', $module['field_security_covered']);
        $node->set('field_module_summary', $module['field_module_summary']);
        $node->set('field_affordances', $module['field_affordances']);
        $node->set('field_module_status', $module['field_module_status']);
        $node->set('field_drupal_author', $module['field_module_author']);

        $node->save();
      }
    }
  }

  /**
   * Download all the schemas.
   *
   * @command project_browser:ai:schema_download
   * @aliases pbsd
   * @usage project_browser:ai:schema_download
   *   Exports the json responses from the project browser endpoint.
   */
  public function downloadSchemas() {
    // Only download from drupal.org - you have to open up the permissions.
    $base = 'https://localhost/drupal-org-proxy/project';
    $query_string['page'] = 0;
    $query_string['limit'] = 24;
    $query_string['sort'] = 'usage_total';
    $query_string['source'] = 'drupalorg_jsonapi';
    $query_string['development_status'] = 'all';
    $query_string['maintenance_status'] = 'maintained';
    $query_string['security_advisory_coverage'] = 'covered';
    $url = $base . '?' . http_build_query($query_string);
    $data = $this->request($url);
    // Get the total pages.
    $total_pages = $data['drupalorg_jsonapi']['totalResults'];
    // Save the first result.
    file_put_contents($this->moduleHandler->getModule('pb_ai')->getPath() . '/downloads/0.json', json_encode($data['drupalorg_jsonapi']));
    // Get the iterations needed.
    $iterations = ceil($total_pages / 24);
    for ($i = 1; $i < $iterations; $i++) {
      $query_string['page'] = $i;
      $url = $base . '?' . http_build_query($query_string);
      $data = $this->request($url);
      echo "Downloaded: " . $i . "\n";
      file_put_contents($this->moduleHandler->getModule('pb_ai')->getPath() . '/downloads/' . $i . '.json', json_encode($data['drupalorg_jsonapi']));
      // Wait 5 seconds to not overload the server.
      sleep(5);
    }
  }

  /**
   * Build module entities from the schemas.
   *
   * @command project_browser:ai:build_modules
   * @aliases pbbm
   * @usage project_browser:ai:build_modules
   *   Creates the node modules from the json schemas.
   */
  public function create_modules() {
    $node_storage = $this->entityTypeManager->getStorage('node');
    // Go through the JSON files one by one.
    $module_path = $this->moduleHandler->getModule('pb_ai')->getPath();
    $files = glob($module_path . '/downloads/*.json');
    foreach ($files as $file) {
      $data = json_decode(file_get_contents($file), TRUE);
      foreach ($data['list'] as $module) {
        // If the package name is drupal/core we skip, since its a recipe.
        if ($module['package_name'] == 'drupal/core') {
          continue;
        }

        // Check if the module exists.
        $nodes = $node_storage->loadByProperties(['field_data_name' => $module['project_machine_name']]);
        if (count($nodes)) {
          // Update.
          $node = $nodes[key($nodes)];
        }
        else {
          // Otherwise create.
          $node = $node_storage->create([
            'type' => 'modules',
            'uid' => User::load(1)->id(),
          ]);
        }
        $node->set('title', $module['title']);
        $node->set('field_module_description', $this->cleanupDescription($module['body']['value']));
        $node->set('field_module_link', 'https://www.drupal.org/project/' . $module['project_machine_name']);
        $node->set('field_creation_date', $module['created']);
        $node->set('field_changed_date', $module['changed']);
        $node->set('field_data_name', $module['project_machine_name']);
        $node->set('field_project_usage', $module['project_usage_total']);
        $node->set('field_security_covered', $module['is_covered'] ? 'not_covered_by_security_advisory' : 'covered_by_security_advisory');
        $node->set('field_drupal_author', $module['author']['name'] ?? '');
        $node->set('field_module_status', $module['status']);
        $node->save();
        echo "Created: " . $module['title'] . "\n";
      }
    }
  }

  /**
   * Resave the data to fill AI fields.
   *
   * @command project_browser:ai:resave_modules
   * @aliases pbrm
   * @usage project_browser:ai:resave_modules
   *   Creates the node modules from the json schemas.
   */
  public function resave_modules() {
    $node_storage = $this->entityTypeManager->getStorage('node');
    $nodes = $node_storage->loadByProperties(['type' => 'modules']);
    foreach ($nodes as $node) {
      // If AI fields are set.
      if ($node->field_module_summary->value && $node->field_affordances->value) {
        continue;
      }
      $node->save();
      echo "Resaved: " . $node->title->value . "\n";
    }
  }

  /**
   * Request the data.
   *
   * @param string $url
   *   The url.
   *
   * @return array
   *   The data.
   */
  protected function request($url) {
    $client = \Drupal::httpClient();
    // User agent client.
    $config['headers']['User-Agent'] = 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/58.0.3029.110 Safari/537.3';
    $response = $client->get($url, $config);
    return json_decode($response->getBody()->getContents(), TRUE);
  }

}
